@@ -257,18 +257,18 @@ class BoundaryAxisBase:
 
         return deriv_evaluator  # type: ignore
 
-    def set_ghost_cells(self, data_all: np.ndarray, *, args=None) -> None:
+    def set_ghost_cells(self, data_full: np.ndarray, *, args=None) -> None:
         """set the ghost cell values for all boundaries
 
         Args:
-            data_all (:class:`~numpy.ndarray`):
+            data_full (:class:`~numpy.ndarray`):
                 The full field data including ghost points
             args:
                 Additional arguments that might be supported by special boundary
                 conditions.
         """
-        self.low.set_ghost_cells(data_all, args=args)
-        self.high.set_ghost_cells(data_all, args=args)
+        self.low.set_ghost_cells(data_full, args=args)
+        self.high.set_ghost_cells(data_full, args=args)
 
     def make_ghost_cell_setter(self) -> GhostCellSetter:
         """return function that sets the ghost cells for this axis on a full array"""
@@ -277,10 +277,10 @@ class BoundaryAxisBase:
         ghost_cell_setter_high = self.high.make_ghost_cell_setter()
 
         @register_jitable
-        def ghost_cell_setter(data_all: np.ndarray, args=None) -> None:
+        def ghost_cell_setter(data_full: np.ndarray, args=None) -> None:
             """helper function setting the conditions on all axes"""
-            ghost_cell_setter_low(data_all, args=args)
-            ghost_cell_setter_high(data_all, args=args)
+            ghost_cell_setter_low(data_full, args=args)
+            ghost_cell_setter_high(data_full, args=args)
 
         return ghost_cell_setter  # type: ignore
 
@@ -348,8 +348,7 @@ class BoundaryPair(BoundaryAxisBase):
             data (str or dict):
                 Data that describes the boundary pair
             rank (int):
-                The tensorial rank of the value associated with the boundary
-                conditions.
+                The tensorial rank of the field for this boundary condition
 
         Returns:
             :class:`~pde.grids.boundaries.axis.BoundaryPair`:
@@ -425,8 +424,8 @@ class BoundaryPair(BoundaryAxisBase):
         """check whether the values at the boundaries have the correct rank
 
         Args:
-            rank (int): The rank of the value that is stored with this
-                boundary condition
+            rank (int):
+                The tensorial rank of the field for this boundary condition
 
         Throws:
             RuntimeError: if the value does not have rank `rank`
@@ -434,11 +433,6 @@ class BoundaryPair(BoundaryAxisBase):
         self.low.check_value_rank(rank)
         self.high.check_value_rank(rank)
 
-    @property
-    def differentiated(self) -> BoundaryPair:
-        """BoundaryPair: differentiated version of this boundary condition"""
-        return self.__class__(self.low.differentiated, self.high.differentiated)
-
 
 class BoundaryPeriodic(BoundaryPair):
     """represent a periodic axis"""
@@ -487,15 +481,10 @@ class BoundaryPeriodic(BoundaryPair):
         """check whether the values at the boundaries have the correct rank
 
         Args:
-            rank (int): The rank of the value that is stored with this
-                boundary condition
+            rank (int):
+                The tensorial rank of the field for this boundary condition
         """
 
-    @property
-    def differentiated(self) -> BoundaryPeriodic:
-        """:class:`BoundaryPeriodic`: differentiated boundary condition"""
-        return self
-
 
 def get_boundary_axis(
     grid: GridBase, axis: int, data, rank: int = 0
@@ -510,8 +499,7 @@ def get_boundary_axis(
         data (str or tuple or dict):
             Data describing the boundary conditions for this axis
         rank (int):
-            The tensorial rank of the value associated with the boundary
-            conditions.
+            The tensorial rank of the field for this boundary condition
 
     Returns:
         :class:`~pde.grids.boundaries.axis.BoundaryAxisBase`:
